/*====================================+=====================================+
! File CApp.h                         ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include <wx/cmdline.h>
#include <wx/app.h>
#include <wx/filename.h>
#include "matelefree.h"
#include "CConfig.h"
#include "CChannel.h"
#include "CPlaylist.h"
#include "matelefree_info.h"
/*-------------------------------------------------------------------------*/
class CFrame ;
class wxImageList ;
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
class CApp : public wxApp
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      ~CApp() ;

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      bool       m_boo_first_start_ever = false ; // Program never started ?
      wxFileName m_fn_executable ; // Current executable
      wxLocale   m_locale ; // Default main locale object
      /*--( Optimize searches by name and num )----------------------------*/
      mtf::t_map_intstr m_map_channel_num ;
      /*-------------------------------------------------------------------*/
      bool       m_boo_forcefreebox = false ;
      /*--( Force smallest mini display ? )--------------------------------*/
      bool       m_boo_force_minimalest = false ;
      wxString   m_s_program_to_start_on_exit ;
      bool       m_boo_use_vlc_init_str = true ;
      bool       m_boo_loading_channels_from_url = false ;
      /*-------------------------------------------------------------------*/
      str_pk_info m_tb_pk_info[ PK_NB ]
      = { { "fbtv", wxTRANSLATE( "TV" ) },                   // PK_FBTV
          { "fbtvb", wxTRANSLATE( "Bouquet basique" ) },     // PK_FBTVB
          { "fbtvo", wxTRANSLATE( "En Option" ) },           // PK_FBTVO
          { "fbtvo_ok", wxTRANSLATE( "En Option accessible" ) },//PK_FBTVO_OK
          { "fbradio", wxTRANSLATE( "Radio" ) },             // PK_FBRADIO
          { "search", wxTRANSLATE( "Search" ) },             // PK_SEARCH
          { "live", wxTRANSLATE( "Live" )},                  // PK_LIVE_NODE
          { CChannelDailymotion::st_get_origin(),            // PK_LIVE_DM
            wxTRANSLATE( "Dailymotion" )
          },
          { CChannelTwitch::st_get_origin(),                 // PK_LIVE_TW
            wxTRANSLATE( "Twitch" )
          },
          { CChannelYoutube::st_get_origin(),                // PK_LIVE_YT
            wxTRANSLATE( "Youtube" )
          },
          { CChannelUstream::st_get_origin(),                // PK_LIVE_US
            wxTRANSLATE( "Ustream" )
          },
          { "user", wxTRANSLATE( "User" ) }                  // PK_USER_NODE
        } ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxFileName m_fn_vlc ; // vlc executable
      /*-------------------------------------------------------------------*/
      mtf::t_map_strstr m_map_image_fn ;
      /*-------------------------------------------------------------------*/
      wxImageList   *m_il_logo = nullptr ;
      t_map_channel m_map_channel ;
      /*-------------------------------------------------------------------*/
      CFrame   *m_frame = nullptr ;
      wxString m_s_media_path ;
      wxString m_s_media_fb_path ;
      wxString m_s_tree_fb_path ;
      wxString m_s_tree_fbtv_path ;
      wxString m_s_tree_live_path ;
      wxString m_s_tree_pl_user_path ;

   /*----------------------------------------------------------------------*/
   public :
      /*--------------------------------------------------------------------+
      ! Serialized member variables : the "m" is upper-case                 !
      +--------------------------------------------------------------------*/
      CConfigRect   M_main_rec ;
      CConfigRect   M_main_mini_rec ;
      CConfigBool   M_boo_mini_display ;
      CConfigBool   M_boo_mini_display_caption ;
      CConfigBool   M_boo_stay_on_top ;
      CConfigString M_s_bopl_sel_name ;
      CConfigBool   M_boo_freebox ;
      CConfigBool   M_boo_mute ;
      CConfigInt    M_i_max_nb_media ;
      CConfigBool   M_boo_new_media_active ;
      CConfigInt    M_i_check_media_delay ;
      CConfigInt    M_i_volume ;
      /*-------------------------------------------------------------------*/
      CConfigRect   M_prefs_rec ;
      CConfigString M_s_lang ; // Canonical name: en_US ...
      CConfigString M_s_vlc_path ;
      CConfigString M_s_snapshot_path ;
      CConfigString M_s_vlc_init_dm ;
      CConfigString M_s_vlc_init_tw ;
      CConfigString M_s_vlc_init_yt ;
      CConfigString M_s_vlc_init_us ;
      CConfigString M_s_vlc_init ;
      CConfigString M_s_deinterlace ;
      CConfigInt    M_i_chkupd_freq ;
      CConfigString M_s_chkupd_last ; // YYYY-MM-DD (ISO)
      CConfigString M_s_def_quality ;
      CConfigInt    M_i_retry_delay ; // Time between two get stream retries
      CConfigBool   M_boo_mute_deact ;
      CConfigBool   M_boo_startup_play ;
      CConfigString M_s_twitch_client_id ;
      CConfigString M_s_proxy_server ;
      CConfigString M_s_startup_play_list ; // Channels at program stop
      CConfigString M_s_startup_play_active ; // Active channel at stop
      CConfigString M_s_play_hist ;
      /*-------------------------------------------------------------------*/
      CConfigRect   M_progress_rec ;
      CConfigRect   M_lastversion_rec ;
      CConfigRect   M_newversion_rec ;
      CConfigRect   M_plname_rec ;
      CConfigRect   M_chname_rec ;
      CConfigRect   M_addlive_rec ;
      /*-------------------------------------------------------------------*/
      CConfigString M_s_main_perspective ;
      CConfigString M_s_bopl_pl_user_list ;
      CConfigString M_s_bopl_expanded ;

   /*----------------------------------------------------------------------*/
   public :
      /*--------------------------------------------------------------------+
      ! Attributes used to display channels and show the played ones.       !
      ! They are defines non here non static to avoid any problem with      !
      ! complex static variables. wxWidget initialization may not have been !
      ! done.                                                               !
      +--------------------------------------------------------------------*/
      wxListItemAttr m_tb_lia[ 2 ] ;
      wxListItemAttr m_tb_lia_active[ 2 ] ;
      wxListItemAttr m_tb_lia_other[ 2 ] ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual bool OnInit() override ;
      virtual void OnInitCmdLine( wxCmdLineParser &parser ) override ;
      virtual bool OnCmdLineParsed( wxCmdLineParser &parser ) override ;
      /*-------------------------------------------------------------------*/
      int actions_for_install_process() ;
      wxString get_exe_path() const { return( m_fn_executable.GetPath() ) ; }
      wxString get_exe_fullpath() const
      {  return( m_fn_executable.GetFullPath() ) ; }
      wxString get_exe_fullname() const
      {  return( m_fn_executable.GetFullName() ) ; }
      bool get_force_minimalest() const { return( m_boo_force_minimalest ) ;}
      bool get_use_vlc_init_str() const { return( m_boo_use_vlc_init_str ) ;}
      CChannel *get_channel( const wxString &s_channel )
      {   auto it = m_map_channel.find( s_channel ) ;
          return( it == m_map_channel.end() ? nullptr : it->second ) ;
      }
      bool get_fb_access() const
      {  return( m_boo_forcefreebox || M_boo_freebox.get() ) ; }
      const wxString &get_program_to_start_on_exit() const
      {  return( m_s_program_to_start_on_exit ) ; }
      void set_program_to_start_on_exit( const wxString &s_program )
      {  m_s_program_to_start_on_exit = s_program ; }
      void restart_on_exit()
      {  m_s_program_to_start_on_exit = get_exe_fullpath() ; }
      void no_start_on_exit()
      {  m_s_program_to_start_on_exit.clear() ; }
      /*-------------------------------------------------------------------*/
      void init_fn_vlc() ;
      void init_imagelist() ;
      void init_translations( const wxString &s_canonical_name
                                           = wxEmptyString
                            ) ;
      CChannel *find_channel( const wxString &s_channel ) const ;
      CChannel *find_channel_num( int i_num ) const ;
      void init_config_info_to_keep() ;
      void init_config_info() ;
      void save_config_info() ;
      /*-------------------------------------------------------------------*/
      int add_channel( CChannel *p_channel ) ;
      int del_channel( const mtf::t_vec_string &vs_channel ) ;
      int rename_channel( const wxString &s_from_full,
                          const wxString &s_to_short
                        ) ;
      int load_channels( const wxString &s_origin, const wxString &s_name,
                         wxInputStream *p_is,
                         mtf::t_vec_string &vs_channel,
                         wxString *p_s_label = nullptr
                       ) ;
      int load_channels_from_file( const wxString &s_filename,
                                   const wxString &s_name,
                                   mtf::t_vec_string &vs_channel,
                                   wxString *p_s_label = nullptr
                                 ) ;
      int load_channels_from_file( const wxString &s_filename,
                                   mtf::t_vec_string &vs_channel
                                 )
      {  return( load_channels_from_file( s_filename, wxEmptyString,
                                          vs_channel
                                        )
               ) ;
      }
      int load_channels_from_url( const wxString &s_url,
                                  const wxString &s_name,
                                  mtf::t_vec_string &vs_channel,
                                  wxString *p_s_label = nullptr
                                ) ;
      int load_channels_freebox_tv( mtf::t_vec_string &vs_channel ) ;
      void init_image_fn() ;
      void init_channels() ;
      void init_playlists() ;
      void load_channel_config_info() ;
      void save_channel_config_info() ;
      void save_playlist_config_info() ;
      wxString next_chkupd( int i_chkupd_freq ) const ;

      /*-------------------------------------------------------------------*/
      const wxString &get_pk_name( int sd )
      {  return( m_tb_pk_info[ sd ].m_s_name ) ; }
      const wxString &get_pk_label( int sd )
      {  return( m_tb_pk_info[ sd ].m_s_label ) ; }
      CPlaylist *get_pk_pl( int sd )
      {  return( m_tb_pk_info[ sd ].m_pl ) ; }
      void set_pk_pl( int sd, CPlaylist *pl )
      {  m_tb_pk_info[ sd ].m_pl = pl ; }
      e_pl_known find_pl_known( const wxString &s ) ;
      bool is_pk_name_reserved( const wxString &s_name ) const ;
      wxString build_vlc_init( CChannel *p_channel ) const ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
DECLARE_APP( CApp )
/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                            End of file CApp.h                             !
+==========================================================================*/
