/*====================================+=====================================+
! File CChannel.h                     ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include <wx/wx.h>
#include <wx/filename.h>
#include "matelefree.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
class CVlcMedia ;
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
struct str_stream
{
   /*----------------------------------------------------------------------*/
   str_stream()
   {  ; }
   str_stream( const wxString &s_name, const wxString &s_url )
            : m_s_name( s_name ), m_s_url( s_url )
   {  ; }
   /*----------------------------------------------------------------------*/
   wxString m_s_name             ; // "HD", "auto", "standard" etc.
   wxString m_s_url              ; // rtsp:// ...
   int      m_i_audio_track = -1 ; // audio track number if >= 0
   mtf::t_vec_string m_vs_option ; // m3u's "#EXTVLCOPT"

   /*----------------------------------------------------------------------*/
   static wxArrayString m_st_as_quality ;
   /*----------------------------------------------------------------------*/
   wxString get_vlc_url() const ;
   /*----------------------------------------------------------------------*/
   friend bool operator==( const str_stream &s1, const wxString &s_name )
   {  return( s1.m_s_name == s_name ) ; }
   /*----------------------------------------------------------------------*/
   static void st_init() ;
   static int st_get_quality_ind( const wxString &s )
   {  return( m_st_as_quality.Index( s ) ) ; }
   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
using t_vec_stream     = std::vector< str_stream >    ;
using t_vec_stream_it  = t_vec_stream::iterator       ;
using t_vec_stream_cit = t_vec_stream::const_iterator ;

/*-------------------------------------------------------------------------*/
class CChannel
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      // CChannel() ;
      virtual ~CChannel()
      {  ; }

      /*-------------------------------------------------------------------*/
      static constexpr const char *m_st_co_p_c_cfg_path = "/channel"  ;

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      wxString m_s_short_name ; // name without source : France 5
      wxString m_s_name       ; // name with source / origin
      wxString m_s_name_find  ; // Prepared name for search
      /*-------------------------------------------------------------------*/
      wxString m_s_logo_fn     ; // Logo file name
      int      m_i_il_pos = -1 ; // Logo pos in image list :
                                 // -1 => Invalid, 0 => Not found
      bool     m_boo_logo_ok = false ;
      /*--( Channel info has to be saved in the config file ? )------------*/
      bool     m_boo_save_cfg = false ;

   /*----------------------------------------------------------------------*/
   protected :
      /*--( Avoid doing things twice (pb context menu calling) )-----------*/
      bool m_boo_getting_streams = false ;
      /*--( Get streams error code )---------------------------------------*/
      enum e_gs_error
      {  GSERR_NO_ERROR, GSERR_DOWNLOAD, GSERR_UNKNOWN_CHANNEL,
         GSERR_PARSING, GSERR_RETRY_LATER, GSERR_NOT_VIEWABLE_HERE,
         GSERR_UNHANDLED, GSERR_NB
      } ;
      e_gs_error m_gs_error = GSERR_NO_ERROR ;

   /*----------------------------------------------------------------------*/
   public :
      /*--( Origin of this channel : fbtv, m3u )---------------------------*/
      wxString     m_s_pl_source ; // fbtv, entv etc.
      e_pl_known   m_pl_known = PK_NB ;
      /*-------------------------------------------------------------------*/
      int          m_i_num = -1 ; // Channel number : "5", "24"
      bool         m_boo_fbtvo = false ;
      t_vec_stream m_vec_stream ;
      /*--( Default track : -1 is used by VLC for "deactivated" )----------*/
      enum { DEFAULT_TRACK = -2 } ;
      /*-------------------------------------------------------------------*/
      wxString     m_s_stream     ; // Active stream
      int          m_i_video_track = DEFAULT_TRACK ;
      int          m_i_audio_track = DEFAULT_TRACK ;
      int          m_i_video_spu   = DEFAULT_TRACK ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      void set_save_cfg( bool boo_val ) { m_boo_save_cfg = boo_val ; }
      bool get_save_cfg() const { return( m_boo_save_cfg ) ; }
      virtual wxString get_origin() const { return( wxEmptyString ) ; }
      const str_stream *get_stream( CVlcMedia *p_media = nullptr ) ;
      void init_streams() ;
      wxString get_short_name() const { return( m_s_short_name ) ; }
      wxString get_name() const { return( m_s_name ) ; }
      wxString get_name_find() const { return( m_s_name_find ) ; }
      wxString build_name( const wxString &s_short_name ) const ;
      void set_short_name( const wxString &s_name ) ;
      wxString get_pl_source() const { return( m_s_pl_source ) ; }
      e_gs_error get_gs_error() const { return( m_gs_error ) ; }
      void set_gs_error( e_gs_error gs_error )
      {  m_gs_error = gs_error ; }
      bool is_gs_retry_later() const
      {  return( m_gs_error == GSERR_RETRY_LATER ) ; }
      bool is_gs_no_error() const
      {  return( m_gs_error == GSERR_NO_ERROR ) ; }
      static wxString st_get_gs_error_message( e_gs_error gse ) ;
      wxString get_gs_error_message() const
      {  return( st_get_gs_error_message( m_gs_error ) ) ; }
      virtual void reset_streams() { ; }
      int get_nb_stream() const { return( m_vec_stream.size() ) ; }
      bool has_stream() const { return( !m_vec_stream.empty() ) ; }
      static wxString st_name_source( const wxString &s_name )
      {  return( s_name.AfterLast( '[' ).BeforeLast( ']' ) ) ; }
      e_pl_known get_pl_known() const { return( m_pl_known ) ; }
      int get_num() const { return( m_i_num ) ; }
      bool is_num_set() const { return( m_i_num > 0 ) ; }
      bool is_fb_num_set() const
      {  return( m_pl_known == PK_FBTV && is_num_set() ) ; }
      bool is_fbtvo() const { return( m_boo_fbtvo ) ; }
      void set_fbtvo( bool boo_val ) { m_boo_fbtvo = boo_val ; }
      wxString get_pl_text_short() const ;
      wxString get_pl_text_long() const ;
      wxString get_title() ;
      void init_logo_fn() ;
      void draw_en_option_overlay( wxBitmap &bmp ) ;
      wxBitmap build_logo() ;
      wxBitmap load_logo() ;
      void init_logo() ;
      int get_logo_il_pos() ;
      wxBitmap get_logo() ;
      wxString datas_to_string() ;
      void string_to_datas( const wxString &s_val ) ;
      void rename( const wxString &s_new_name ) ;
      virtual wxString get_url() const { return( wxEmptyString ) ; }
      virtual wxString get_vlc_init() const ;
      /*--( Only really used for streams )---------------------------------*/
      void message_get_all_streams( CVlcMedia *p_media = nullptr ) ;
      virtual int get_all_streams( CVlcMedia *p_media = nullptr )
      {  return( 0 ) ; }

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CChannelLive : public CChannel
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CChannelLive( const wxString &s_id, const wxString &s_name )
                  : m_s_id( s_id )
      {  m_pl_known = PK_LIVE_NODE ; }
      virtual ~CChannelLive()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      static constexpr const char *m_st_co_p_c_id_example = "CHANNEL_ID" ;
      /*-------------------------------------------------------------------*/
      wxString m_s_id ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual wxString get_origin() const = 0 ;
      wxString get_id() const { return( m_s_id ) ; }
      void set_id( const wxString &s_id ) { m_s_id = s_id ; }
      virtual wxString get_url_format() const = 0 ;
      virtual wxString get_url_example() const = 0 ;
      virtual wxString get_url() const
      {  return( wxString::Format( get_url_format(), m_s_id ) ) ; }
      /*--( Next play will force streams reload: useful when Access Error )*/
      virtual void reset_streams()
      {  m_vec_stream.clear() ; }
      /*-------------------------------------------------------------------*/
      int hls_extract_streams( const wxString &s_url_m3u,
                               bool boo_reverse = false
                             ) ;
   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CChannelDailymotion : public CChannelLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CChannelDailymotion( const wxString &s_id, const wxString &s_name )
                         : CChannelLive( s_id, s_name )
      {  /*----------------------------------------------------------------*/
         m_s_pl_source = get_origin() ;
         set_short_name( s_name ) ;
         /*----------------------------------------------------------------*/
      }
      virtual ~CChannelDailymotion()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      static wxString st_get_origin()
      {  return( "dailymotion" ) ; }
      virtual wxString get_origin() const override
      {  return( st_get_origin() ) ; }
      static wxString st_get_url_format()
      {  return( "http://www.dailymotion.com/video/%s" ) ; }
      static wxString st_build_url( const wxString &s_id )
      {  return( wxString::Format( st_get_url_format(), s_id ) ) ; }
      static wxString st_get_url_example()
      {  return( st_build_url( m_st_co_p_c_id_example ) ) ; }
      virtual wxString get_url_format() const override
      {  return( st_get_url_format() ) ; }
      virtual wxString get_url_example() const override
      {  return( st_get_url_example() ) ; }
      /*-------------------------------------------------------------------*/
      static int st_extract_data_from_url( const wxString &s_url,
                                           wxString &s_id,
                                           wxString &s_name,
                                           wxString *p_s_err_msg
                                         ) ;
      /*-------------------------------------------------------------------*/
      virtual wxString get_vlc_init() const override ;
      virtual int get_all_streams( CVlcMedia *p_media = nullptr ) override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CChannelTwitch : public CChannelLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CChannelTwitch( const wxString &s_id, const wxString &s_name )
                    : CChannelLive( s_id, s_name )
      {  /*----------------------------------------------------------------*/
         m_s_pl_source = get_origin() ;
         set_short_name( s_name ) ;
         /*----------------------------------------------------------------*/
      }
      virtual ~CChannelTwitch()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*--( Time after which the streams are not supposed to be valid )----*/
      time_t m_ti_expiration = 0 ;

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      wxString url_encode( const wxString &s ) ;
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      static wxString st_get_origin()
      {  return( "twitch" ) ; }
      virtual wxString get_origin() const override
      {  return( st_get_origin() ) ; }
      static wxString st_get_url_format()
      {  return( "http://www.twitch.tv/%s" ) ; }
      static wxString st_build_url( const wxString &s_id )
      {  return( wxString::Format( st_get_url_format(), s_id ) ) ; }
      static wxString st_get_url_example()
      {  return( st_build_url( m_st_co_p_c_id_example ) ) ; }
      virtual wxString get_url_format() const override
      {  return( st_get_url_format() ) ; }
      virtual wxString get_url_example() const override
      {  return( st_get_url_example() ) ; }
      /*-------------------------------------------------------------------*/
      static e_gs_error st_get_token_sig( const wxString &s_id,
                                          wxString *p_s_token,
                                          wxString *p_s_sig
                                        ) ;
      static int st_extract_data_from_url( const wxString &s_url,
                                           wxString &s_id,
                                           wxString &s_name,
                                           wxString *p_s_err_msg
                                         ) ;
      /*-------------------------------------------------------------------*/
      virtual wxString get_vlc_init() const override ;
      virtual int get_all_streams( CVlcMedia *p_media = nullptr ) override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CChannelYoutube : public CChannelLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CChannelYoutube( const wxString &s_id, const wxString &s_name )
                     : CChannelLive( s_id, s_name )
      {  /*----------------------------------------------------------------*/
         m_s_pl_source = get_origin() ;
         set_short_name( s_name ) ;
         /*----------------------------------------------------------------*/
      }
      virtual ~CChannelYoutube()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      static wxString st_get_origin()
      {  return( "youtube" ) ; }
      virtual wxString get_origin() const override
      {  return( st_get_origin() ) ; }
      static wxString st_get_url_format()
      {  return( "https://www.youtube.com/channel/%s" ) ; }
      static wxString st_build_url( const wxString &s_id )
      {  return( wxString::Format( st_get_url_format(), s_id ) ) ; }
      static wxString st_get_url_example()
      {  return( st_build_url( m_st_co_p_c_id_example ) ) ; }
      virtual wxString get_url_format() const override
      {  return( st_get_url_format() ) ; }
      virtual wxString get_url_example() const override
      {  return( st_get_url_example() ) ; }
      /*-------------------------------------------------------------------*/
      static int st_extract_data_from_url( const wxString &s_url,
                                           wxString &s_id,
                                           wxString &s_name,
                                           wxString *p_s_err_msg
                                         ) ;
      /*-------------------------------------------------------------------*/
      virtual wxString get_vlc_init() const override ;
      virtual int get_all_streams( CVlcMedia *p_media = nullptr ) override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CChannelUstream : public CChannelLive
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CChannelUstream( const wxString &s_id, const wxString &s_name )
                     : CChannelLive( s_id, s_name )
      {  /*----------------------------------------------------------------*/
         m_s_pl_source = get_origin() ;
         set_short_name( s_name ) ;
         /*----------------------------------------------------------------*/
      }
      virtual ~CChannelUstream()
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxString m_s_username ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      static wxString st_get_origin()
      {  return( "ustream" ) ; }
      virtual wxString get_origin() const override
      {  return( st_get_origin() ) ; }
      /*--------------------------------------------------------------------+
      ! As :                                                                !
      ! 1) for Ustream "get_url()" is only used to "open the browser"       !
      ! 2) only the video id is known (and not the channel id)              !
      ! 3) I don't know how to convert the video id to the channel one      !
      ! Then : the browser URL is not the "channel" page but the embedded   !
      ! video page                                                          !
      +--------------------------------------------------------------------*/
      static wxString st_get_url_format()
      {  return( "http://www.ustream.tv/%s" ) ; }
      static wxString st_build_url( const wxString &s_id )
      {  return( wxString::Format( st_get_url_format(), s_id ) ) ; }
      static wxString st_get_url_example()
      {  return( st_build_url( "USERNAME" ) ) ; }
      virtual wxString get_url_format() const override
      {  return( st_get_url_format() ) ; }
      virtual wxString get_url_example() const override
      {  return( st_get_url_example() ) ; }
      /*-------------------------------------------------------------------*/
      virtual wxString get_url() const override
      {  return( st_build_url( m_s_username ) ) ; }
      static int st_extract_data_from_url( const wxString &s_url,
                                           wxString &s_id,
                                           wxString &s_name,
                                           wxString *p_s_err_msg
                                         ) ;
      /*-------------------------------------------------------------------*/
      virtual wxString get_vlc_init() const override ;
      virtual int get_all_streams( CVlcMedia *p_media = nullptr ) override ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
using t_map_channel     = std::map< wxString, CChannel * > ;
using t_map_channel_it  = t_map_channel::iterator          ;
using t_map_channel_cit = t_map_channel::const_iterator    ;
/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                          End of file CChannel.h                           !
+==========================================================================*/
