/*====================================+=====================================+
! File CDlgPrefs.h                    ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include <wx/bookctrl.h>
#include <wx/propdlg.h>
#include <wx/spinctrl.h>
#include "CVlcInterface.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
class CDlgPrefs ;
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
class CPanPrefs : public wxPanel
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPanPrefs( wxWindow *parent, CDlgPrefs *dlg )
               : wxPanel( parent ), m_dlg( dlg )
      {  ; }

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CDlgPrefs *m_dlg ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      virtual void init_default() = 0 ;
      virtual bool apply_changes() = 0 ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPanPrefsGen : public CPanPrefs
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPanPrefsGen( wxWindow *parent, CDlgPrefs *dlg ) ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxStaticText *m_stx_vlc_info ;
      wxTextCtrl   *m_txt_vlc_path ;
      wxButton     *m_bt_vlc_path_dlg ;
      wxTextCtrl   *m_txt_snapshot_path ;
      wxButton     *m_bt_snapshot_path_dlg ;
      wxChoice     *m_cho_lang ;
      wxCheckBox   *m_chk_freebox ;
      wxCheckBox   *m_chk_startup_play ;
      wxChoice     *m_cho_chkupd_freq ;
      wxStaticText *m_stx_chkupd_next ;
      wxChoice     *m_cho_def_quality ;
      wxSpinCtrl   *m_i_max_nb_media ;
      wxCheckBox   *m_chk_new_media_active ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      void OnVlcPathChanged( wxCommandEvent &event ) ;
      void OnSnapshotPathChanged( wxCommandEvent &event ) ;
      void OnChooseVlcPath( wxCommandEvent &event ) ;
      void OnChooseSnapshotPath( wxCommandEvent &event ) ;
      /*-------------------------------------------------------------------*/
      void build_ui() ;
      /*-------------------------------------------------------------------*/
      virtual void init_default() override ;
      virtual bool apply_changes() override ;
      /*-------------------------------------------------------------------*/
      wxString get_vlc_path() const
      {  return( m_txt_vlc_path->GetValue() ) ; }
      wxString get_snapshot_path() const
      {  return( m_txt_snapshot_path->GetValue() ) ; }
      void check_vlc_path() ;
      void append_cho_lang( const wxString &s_desc,
                            const wxString &s_canon
                          ) ;
      wxString get_cho_lang_sel_canon() ;
      void init_cho_lang() ;
      void init_chkupd_next() ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPanPrefsAdv : public CPanPrefs
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPanPrefsAdv( wxWindow *parent, CDlgPrefs *dlg ) ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxTextCtrl *m_txt_vlc_init_dm ;
      wxTextCtrl *m_txt_vlc_init_tw ;
      wxTextCtrl *m_txt_vlc_init_yt ;
      wxTextCtrl *m_txt_vlc_init_us ;
      wxTextCtrl *m_txt_vlc_init ;
      wxChoice   *m_cho_deinterlace ;
      wxCheckBox *m_chk_mute_deact ;
      wxSpinCtrl *m_i_check_media_delay ;
      wxSpinCtrl *m_i_retry_delay ;
      wxTextCtrl *m_txt_twitch_client_id ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      void build_ui() ;
      /*-------------------------------------------------------------------*/
      virtual void init_default() ;
      virtual bool apply_changes() ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CPanPrefsNet : public CPanPrefs
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CPanPrefsNet( wxWindow *parent, CDlgPrefs *dlg ) ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxTextCtrl *m_txt_proxy_server ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      void build_ui() ;
      /*-------------------------------------------------------------------*/
      virtual void init_default() ;
      virtual bool apply_changes() ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CDlgPrefs : public wxPropertySheetDialog
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CDlgPrefs() ;
      ~CDlgPrefs() ;

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      CPanPrefsGen *m_p_gen = nullptr ; // To check vlc path validity

   /*----------------------------------------------------------------------*/
   public :
      /*--( Some changes need a program restart to be active )-------------*/
      bool m_boo_restart = false ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      void OnBtOk( wxCommandEvent &event )
      {  apply_changes() ; EndModal( event.GetId() ) ; }
      void OnBtApply( wxCommandEvent &event )
      {  apply_changes() ; }
      void OnBtResetPage( wxCommandEvent &event )
      {  get_current_page()->init_default() ; }
      /*-------------------------------------------------------------------*/
      size_t get_nb_page() const
      {  return( GetBookCtrl()->GetPageCount() ) ; }
      /*-------------------------------------------------------------------*/
      CPanPrefs *get_page( size_t sz_num )
      {  return( static_cast<CPanPrefs *>( GetBookCtrl()->GetPage( sz_num ) )
               ) ;
      }
      /*-------------------------------------------------------------------*/
      CPanPrefs *get_current_page()
      {  return( get_page( GetBookCtrl()->GetSelection() ) ) ; }
      /*-------------------------------------------------------------------*/
      void build_ui() ;
      void apply_changes() ;
      int get_modal() ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                          End of file CDlgPrefs.h                          !
+==========================================================================*/
