/*====================================+=====================================+
! File CFrame.h                       ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#pragma once
/*-------------------------------------------------------------------------*/
#include <wx/wx.h>
#include <wx/frame.h>
#include <wx/taskbar.h>
#include <wx/aui/framemanager.h>
#include <wx/aui/auibar.h>
#include <wx/wrapsizer.h>
#include "CVlcInterface.h"
#include "CVlcMedia.h"
#include "CPlaylist.h"
#include "CBookPlaylist.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
class CTaskBarIcon : public wxTaskBarIcon
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CTaskBarIcon() ;
      //~CTaskBarIcon() ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      void OnLeftDown( wxTaskBarIconEvent &event ) ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/
class CFrame : public wxFrame
{
   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      CFrame() ;
      ~CFrame() ;

   /*----------------------------------------------------------------------*/
   private :
      /*-------------------------------------------------------------------*/
      const int co_i_volmax = 150 ;
      /*-------------------------------------------------------------------*/
      wxMenuBar      *m_menubar      = nullptr ;
      CTaskBarIcon   *m_taskbar_icon = nullptr ;
      /*-------------------------------------------------------------------*/
      wxPanel        *m_panel_media  = nullptr ;
      wxBoxSizer     *m_bsz_media    = nullptr ;
      wxPanel        *m_panel_ctrl   = nullptr ;
      wxBoxSizer     *m_bsz_ctrl     = nullptr ;
      wxAuiToolBar   *m_to_ctrl      = nullptr ;
      wxSlider       *m_sld_volume   = nullptr ;
      /*-------------------------------------------------------------------*/
      wxMenu         *m_menu_play_hist = nullptr ;
      mtf::t_deq_string m_ds_play_hist ; // Play history
      /*-------------------------------------------------------------------*/
      t_vec_vlc_media m_vec_media ;
      CVlcMedia       *m_p_media_active = nullptr ;
      /*-------------------------------------------------------------------*/
      wxTimer         m_timer_mouse_off ;
      bool            m_boo_first_idle_done = false ;
      /*-------------------------------------------------------------------*/
      wxString        m_s_before_fullscreen_perspective ;
      e_fullscreen_level m_fullscreen_level = FSL_NONE ;
      /*--------------------------------------------------------------------+
      ! On this system, can the mini display be the smallest possible ? :   !
      ! - No menu                                                           !
      ! - No caption                                                        !
      ! A "test" is done during startup                                     !
      +--------------------------------------------------------------------*/
      bool            m_boo_mini_display_smallest = false ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxAuiManager   m_auimgr ;
      CBookPlaylist  *m_bopl      = nullptr ;
      wxMenu         *m_menu_play = nullptr ;

   /*----------------------------------------------------------------------*/
   public :
      /*-------------------------------------------------------------------*/
      wxMenuBar *get_menubar()
      {  return( m_menubar ) ; }
      bool get_mini_display_smallest() const
      {  return( m_boo_mini_display_smallest ) ; }
      const t_vec_vlc_media &get_vec_media() const
      {  return( m_vec_media ) ; }
      /*-------------------------------------------------------------------*/
      void init() ;
      void build_menubar() ;
      void OnFirstIdle( wxIdleEvent &event ) ;
      void OnPrefs( wxCommandEvent &event ) ;
      void OnBtPlayStop( wxAuiToolBarEvent &event ) ;
      void OnBtPlayHist( wxAuiToolBarEvent &event ) ;
      void OnAbout( wxCommandEvent & WXUNUSED( event ) ) ;
      void OnFullScreenMouseOn( wxMouseEvent &event ) ;
      void OnFullScreenMouseOff( wxTimerEvent &event ) ;
      void OnSldVolumeMouseWheel( wxMouseEvent &event ) ;
      /*-------------------------------------------------------------------*/
      wxMenuBar *get_menubar() const
      {  return( m_menubar ) ; }
      wxMenu *get_menu_play() const
      {  return( m_menu_play ) ; }
      wxWindow *get_window_pl()
      {  return( m_bopl->get_window_pl() ) ; }
      wxSizer *get_sizer_media() const
      {  return( m_bsz_media ) ; }
      wxPanel *get_panel_media() const
      {  return( m_panel_media ) ; }
      CVlcMedia *get_media_active()
      {  return( m_p_media_active ) ; }
      bool get_first_idle_done() const
      {  return( m_boo_first_idle_done ) ; }
      e_fullscreen_level get_fullscreen_level() const
      {  return( m_fullscreen_level ) ; }
      void set_title() ;
      void save_display_state() ;
      void setup_display_rect() ;
      void setup_display_mode() ;
      void play_on_start() ;
      void create_vlc_interface() ;
      void delete_vlc_interface() ;
      CPlaylistUser *create_new_pl_user() ;
      CVlcMedia *new_media( const wxString &s_for_channel ) ;
      void delete_media( CVlcMedia *p_media ) ;
      int reinit_current_media( const wxString &s_for_channel ) ;
      void layout_media() ;
      void set_media_active( CVlcMedia *p_media ) ;
      bool set_media_active( size_t sz_num ) ;
      bool set_media_active( const wxString &s_channel ) ;
      int find_media( const CVlcMedia *p_media ) ;
      CVlcMedia *find_media( const CChannel *p_channel ) ;
      CVlcMedia *find_media( const wxString &s_channel ) ;
      bool check_nb_media( int i_tot_nb ) ;
      void play_channel( const wxString &s_channel,
                         bool boo_new_media = false
                       ) ;
      void play_channel( const mtf::t_vec_string &vs_channel,
                         const wxString &s_active = wxEmptyString
                       ) ;
      int nb_media_to_show() const
      {  return( m_fullscreen_level == FSL_ONE_MEDIA
                 ? wxMin( m_vec_media.size(), 1 ) : m_vec_media.size()
               ) ;
      }
      int close_media( CVlcMedia *p_media, bool boo_layout ) ;
      void close_media_active()
      {  close_media( get_media_active(), true ) ; }
      void close_all_media() ;
      void refresh_menu_play_hist() ;
      CPlaylist *get_playlist_active()
      {  return( m_bopl->get_playlist_active() ) ; }
      void toggle_mute() ;
      void set_volume_from_slider() ;
      void toggle_pane_show( wxWindow *win_pane ) ;
      void init_pane_floating_size( wxWindow *win_pane ) ;
      void init_gui_pane() ;
      void init_gui_play_hist() ;
      void init_gui_play() ;
      void init_gui_mini_display() ;
      void init_gui_fullscreen() ;
      void init_gui_mute() ;
      void play_start_pause() ;
      void play_focus() ;
      void play_prev_hil() ;
      void play_next_hil() ;
      void play_hist() ;
      void play_hist_clear() ;
      void toggle_mini_display() ;
      void toggle_show() ;
      void toggle_caption() ;
      void set_fullscreen( e_fullscreen_level fsl ) ;
      void next_fullscreen() ;
      void add_play_hist( const CChannel *p_channel ) ;
      void take_snapshot() ;
      /*-------------------------------------------------------------------*/
      void check_version( int i_err_code, const wxString &s_data,
                          bool boo_check_manual
                        ) ;
      wxString get_play_hist() ;
      void set_play_hist( const wxString &s_play_hist ) ;

   /*----------------------------------------------------------------------*/
} ;

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                           End of file CFrame.h                            !
+==========================================================================*/
