/*====================================+=====================================+
! File CChannel.cpp                   ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+-------+-------------------------------------------------------------------+
! Notes !                                                                   !
+-------+                                                                   !
!                                                                           !
! Twitch parsing code bellow is based on :                                  !
! Twitch.tv playlist parser v0.0.4 by Stefan Sundin                         !
!                                                                           !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include <wx/clipbrd.h>
#include <wx/filename.h>
#include <wx/textctrl.h>
#include <wx/stattext.h>
#include <wx/button.h>
#include <wx/dir.h>
#include <wx/sstream.h>
#include <wx/txtstrm.h>
#include <wx/regex.h>
#include <wx/imaglist.h>
#include <curl/curl.h>
#include "matelefree.h"
#include "matelefree_info.h"
#include "CApp.h"
#include "CVlcInterface.h"
#include "CDownload.h"
#include "CFrame.h"
#include "CPlaylist.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
wxArrayString str_stream::m_st_as_quality ;

/*-------------------------------------------------------------------------*/
wxString str_stream::get_vlc_url() const
{
   /*----------------------------------------------------------------------*/
   wxString s_url = m_s_url ;

   /*--( Adding the VLC options )------------------------------------------*/
   for( const wxString &s_option : m_vs_option )
   {  s_url += " :" + s_option ; }

   /*----------------------------------------------------------------------*/
   return( s_url ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void str_stream::st_init()
{
   /*-----------------------------------------------------------------------+
   ! The different stream qualities as defined by the "Freebox"             !
   ! This will be present in prefs and used to sort channel streams         !
   +-----------------------------------------------------------------------*/
   m_st_as_quality.Add( "auto" ) ;
   m_st_as_quality.Add( wxString::FromUTF8( u8"bas d\u00e9bit" ) ) ;
   m_st_as_quality.Add( "standard" ) ;
   m_st_as_quality.Add( "HD" ) ;
   m_st_as_quality.Add( "TNT" ) ;
   m_st_as_quality.Add( "TNT HD" ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CChannel::build_name( const wxString &s_short_name ) const
{
   /*----------------------------------------------------------------------*/
   return(   s_short_name
           + " ["
           + ( m_pl_known == PK_LIVE_NODE ? get_origin() : get_pl_source() )
           + ']'
         ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CChannel::set_short_name( const wxString &s_name )
{
   /*----------------------------------------------------------------------*/
   m_s_short_name = s_name ;
   m_s_name_find = mtf::prepare_string( m_s_short_name ) ;
   m_s_name = build_name( m_s_short_name ) ;
   /*--( The logo is based on the channel short name )---------------------*/
   init_logo_fn() ;
   /*--( The logo has to be recreated )------------------------------------*/
   m_boo_logo_ok = false ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CChannel::st_get_gs_error_message( e_gs_error gse )
{
   /*----------------------------------------------------------------------*/
   switch( gse )
   {
      /*-------------------------------------------------------------------*/
      case GSERR_NO_ERROR : return( wxEmptyString ) ;
      case GSERR_DOWNLOAD : return( _( "Problem downloading data" ) ) ;
      case GSERR_UNKNOWN_CHANNEL : return( _( "Unknown channel" ) ) ;
      case GSERR_PARSING : return( _( "Information parsing error" ) ) ;
      case GSERR_RETRY_LATER : return( wxEmptyString ) ;
      case GSERR_NOT_VIEWABLE_HERE :
         return( wxString::Format( _( "For copyright reasons this channel\n"
                                      "can't be played by an application "
                                      "like %s"

                                    ),
                                   MTF_DISPLAY_NAME
                                 )
               ) ;
      /*-------------------------------------------------------------------*/
      case GSERR_UNHANDLED : return( _( "Unhandled error" ) ) ;
      /*-------------------------------------------------------------------*/
      default :
         return( wxString::Format( _( "Unknown error code (%s)" ), gse ) ) ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CChannel::get_pl_text_short() const
{
   /*----------------------------------------------------------------------*/
   if( !is_num_set() ) { return( get_short_name() ) ; }
   /*----------------------------------------------------------------------*/
   return( wxString::Format( "%d - %s", m_i_num, get_short_name() ) ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CChannel::get_pl_text_long() const
{
   /*----------------------------------------------------------------------*/
   if( !is_num_set() ) { return( get_name() ) ; }
   /*----------------------------------------------------------------------*/
   return( wxString::Format( "%d - %s", m_i_num, get_name() ) ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CChannel::get_title()
{
   /*----------------------------------------------------------------------*/
   wxString s_title = get_pl_text_long() ;
   /*--( Stream info is valuable only if more than one is available )------*/
   if( m_vec_stream.size() > 1 )
   {  s_title += " (" + get_stream()->m_s_name + ")" ; }
   /*----------------------------------------------------------------------*/
   return( s_title ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CChannel::init_logo_fn()
{
   /*-----------------------------------------------------------------------+
   ! How to find the channel logo name ?                                    !
   ! The search is based on the channel name.                               !
   !                                                                        !
   ! How is structured a channel name ? : channel_base_name (stream name)   !
   ! For example, the following channels will share the same logo :         !
   ! Arte (HD)                                                              !
   ! Arte (francais)                                                        !
   !                                                                        !
   ! The text between '(' and ')', the stream name, must be at the end.     !
   ! The logo search is done from the start of the name to the " (" of      !
   ! the stream name.                                                       !
   !                                                                        !
   ! To avoid issues with the OS naming restrictions the search is done     !
   ! after making the following changes :                                   !
   ! - Unaccent                                                             !
   ! - Lowercase                                                            !
   ! - Replace "special" characters with '-'                                !
   +-----------------------------------------------------------------------*/
   mtf::t_map_strstr_cit cit_name
                       = wxGetApp().m_map_image_fn.find( get_name_find() ) ;
   /*----------------------------------------------------------------------*/
   if( cit_name == wxGetApp().m_map_image_fn.end() )
   {  m_s_logo_fn.clear() ; }
   else
   {  /*--( No swap because different channels can have same names )-------*/
      m_s_logo_fn = cit_name->second ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*--( Add "en option" channel logo detail )--------------------------------*/
void CChannel::draw_en_option_overlay( wxBitmap &bmp )
{
   /*--( The "bmp" parameter is not supposed to have transparent points )--*/
   wxMemoryDC dc( bmp ) ;
   wxColour   colour_gold( 255, 215, 0 ) ;
   wxColour   colour_goldenrod( 218, 165, 32 ) ;
   wxBrush    brush_gold( colour_gold ) ;
   wxPen      pen_gold( colour_goldenrod ) ;
   /*--( Draw upper left triangle )----------------------------------------*/
   dc.SetBrush( brush_gold ) ;
   dc.SetPen( pen_gold ) ;
   wxPoint tb_pt[] = { { 0, 0 }, { 15, 0 }, { 0, 15 } } ;
   dc.DrawPolygon( WXSIZEOF( tb_pt ), tb_pt ) ;
   /*--( Draw frame )------------------------------------------------------*/
   dc.SetBrush( *wxTRANSPARENT_BRUSH ) ;
   dc.DrawRectangle( { 0, 0 }, bmp.GetSize() ) ;
   dc.SelectObject( wxNullBitmap ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxBitmap CChannel::build_logo()
{
   /*----------------------------------------------------------------------*/
   wxString s_name = get_short_name() ;
   size_t sz_name = s_name.size() ;
   wxUint8 tb_b_rgb[ 3 ] ;
   /*--( Background colour computed based on text )------------------------*/
   tb_b_rgb[ 0 ] = tb_b_rgb[ 1 ] = tb_b_rgb[ 2 ] = 0xAA ;
   for( size_t sz_num = 0 ; sz_num < sz_name ; ++sz_num )
   {  tb_b_rgb[ sz_num % 3 ] ^= ( s_name[ sz_num ].GetValue() << 4 ) ; }
   wxColour color_background( tb_b_rgb[ 0 ], tb_b_rgb[ 1 ], tb_b_rgb[ 2 ] ) ;
   /*--( Foreground colour computed based on the background one )----------*/
   double do_brightness
    = tb_b_rgb[ 0 ] * 0.299 + tb_b_rgb[ 1 ] * 0.587 + tb_b_rgb[ 2 ] * 0.114 ;
   wxColour color_foreground( do_brightness > 160 ? *wxBLACK : *wxWHITE ) ;
   /*----------------------------------------------------------------------*/
   wxBitmap bmp( wxGetApp().m_il_logo->GetSize() ) ;
   /*----------------------------------------------------------------------*/
   wxMemoryDC dc( bmp ) ;
   dc.SetBackground( color_background ) ;
   dc.Clear() ;
   dc.SetFont( wxSystemSettings::GetFont( wxSYS_DEFAULT_GUI_FONT ) ) ;
   dc.SetTextForeground( color_foreground ) ;
   /*----------------------------------------------------------------------*/
   mtf::draw_wrapped( dc, s_name ) ;
   dc.SelectObject( wxNullBitmap ) ;
   /*----------------------------------------------------------------------*/
   return( bmp ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxBitmap CChannel::load_logo()
{
   /*----------------------------------------------------------------------*/
   if( m_s_logo_fn.empty() ) { return( wxBitmap() ) ; }
   /*----------------------------------------------------------------------*/
   wxLogNull no_log ; // Suppress useless warnings
   wxBitmap  bmp( m_s_logo_fn, wxBITMAP_TYPE_PNG ) ;
   /*--( Invalid file or invalid image size )------------------------------*/
   if( !bmp.IsOk() || bmp.GetSize() != wxGetApp().m_il_logo->GetSize() )
   {  bmp = build_logo() ; }
   else
   if( bmp.HasAlpha() )
   {
      /*--------------------------------------------------------------------+
      ! Remove alpha channel to improve logo display "quality" on different !
      ! backgrounds : channel list and media message                        !
      ! The playlist active record looks nicer                              !
      +--------------------------------------------------------------------*/
      wxBitmap bmp_no_alpha( bmp.GetSize() ) ;
      /*-------------------------------------------------------------------*/
      wxMemoryDC dc( bmp_no_alpha ) ;
      dc.SetBackground( wxSystemSettings::GetColour( wxSYS_COLOUR_WINDOW ) );
      dc.Clear() ;
      dc.DrawBitmap( bmp, 0, 0 ) ;
      dc.SelectObject( wxNullBitmap ) ;
      /*-------------------------------------------------------------------*/
      bmp = bmp_no_alpha ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return( bmp ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CChannel::init_logo()
{
   /*--( If the file is unknown the logo is "built" )----------------------*/
   wxBitmap bmp = ( m_s_logo_fn.empty() ? build_logo() : load_logo() ) ;
   /*--( Add overlay for freebox "en option" channels )--------------------*/
   if( is_fbtvo() ) { draw_en_option_overlay( bmp ) ; }
   /*--( Replace or add logo ? )-------------------------------------------*/
   if( m_i_il_pos < 0 )
   {  m_i_il_pos = wxGetApp().m_il_logo->Add( bmp ) ; }
   else
   {  wxGetApp().m_il_logo->Replace( m_i_il_pos, bmp ) ; }
   /*----------------------------------------------------------------------*/
   m_boo_logo_ok = true ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CChannel::get_logo_il_pos()
{
   /*----------------------------------------------------------------------*/
   if( !m_boo_logo_ok ) { init_logo() ; }
   /*----------------------------------------------------------------------*/
   return( m_i_il_pos ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxBitmap CChannel::get_logo()
{
   /*----------------------------------------------------------------------*/
   return( wxGetApp().m_il_logo->GetBitmap( get_logo_il_pos() ) ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
const str_stream *CChannel::get_stream( CVlcMedia *p_media )
{
   /*--( Do we have to "reload" the stream urls ? )------------------------*/
   if( get_all_streams( p_media ) != 0 || get_nb_stream() == 0 )
   {  return( nullptr ) ; }
   /*--( Search for the "active" stream or, by default, the first one )----*/
   t_vec_stream_cit it ;
   it = ( m_s_stream.empty()
          ? m_vec_stream.end()
          : std::find( m_vec_stream.begin(), m_vec_stream.end(), m_s_stream )
        ) ;
   /*--( By default the current stream is the first one )------------------*/
   if( it == m_vec_stream.end() )
   {  it = m_vec_stream.begin() ;
      m_s_stream = it->m_s_name ;
   }
   /*----------------------------------------------------------------------*/
   return( &*it ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CChannel::init_streams()
{
   /*----------------------------------------------------------------------*/
   if( m_vec_stream.empty() ) { return ; }
   /*--( Streams sort and default : we are only sure of the freebox ones )-*/
   if( m_pl_known != PK_FBTV )
   {  m_s_stream = m_vec_stream[ 0 ].m_s_name ; }
   else
   {
      /*-------------------------------------------------------------------*/
      std::sort( m_vec_stream.begin(), m_vec_stream.end(),
                 []( const str_stream &s1, const str_stream &s2 )
                 {  return(   str_stream::st_get_quality_ind( s1.m_s_name )
                            < str_stream::st_get_quality_ind( s2.m_s_name )
                          ) ;
                 }
               ) ;
      /*--( The default quality is searched based on "Preferences" )-------*/
      wxString s_quality = ( wxGetApp().M_s_def_quality.empty()
                             ? "auto" : wxGetApp().M_s_def_quality.get()
                           ) ;
      t_vec_stream_it it_stream
      = std::find_if( m_vec_stream.begin(), m_vec_stream.end(),
                      [ &s_quality ]( const str_stream &stream )
                      {  return( stream.m_s_name == s_quality ) ; }
                    ) ;
      if( it_stream != m_vec_stream.end() )
      {  m_s_stream = it_stream->m_s_name ; }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CChannel::datas_to_string()
{
   /*----------------------------------------------------------------------*/
   return( wxString::Format( "\"%s\" %d %d %d",
                             get_stream()->m_s_name,
                             m_i_video_track, m_i_audio_track, m_i_video_spu
                           )
         ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CChannel::string_to_datas( const wxString &s_val )
{
   /*----------------------------------------------------------------------*/
   char tb_c_stream_name[ 256 ] ;
   int  i_video_track = DEFAULT_TRACK ;
   int  i_audio_track = DEFAULT_TRACK ;
   int  i_video_spu   = DEFAULT_TRACK ;
   /*----------------------------------------------------------------------*/
   wxSscanf( s_val, "\"%255[^\"]\" %d %d %d", tb_c_stream_name,
             &i_video_track, &i_audio_track, &i_video_spu
           ) ;
   tb_c_stream_name[ WXSIZEOF( tb_c_stream_name ) - 1 ] = '\0' ;

   /*----------------------------------------------------------------------*/
   m_s_stream      = tb_c_stream_name ;
   m_i_video_track = i_video_track ;
   m_i_audio_track = i_audio_track ;
   m_i_video_spu   = i_video_spu   ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CChannel::rename( const wxString &s_new_name )
{
   /*----------------------------------------------------------------------*/
   set_short_name( s_new_name ) ;
   init_logo() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CChannel::get_vlc_init() const
{  return( wxGetApp().M_s_vlc_init.get() ) ; }

/*-------------------------------------------------------------------------*/
void CChannel::message_get_all_streams( CVlcMedia *p_media )
{
   /*----------------------------------------------------------------------*/
   if( p_media == nullptr ) { return ; }
   /*----------------------------------------------------------------------*/
   p_media->set_message( _( "Looking for streams ..." ) ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
static int st_extract_name_from_page( const wxString &s_data,
                                      wxString &s_name,
                                      wxString *p_s_err_msg
                                    )
{  /*--( Manage different arrangements of "meta property content" )--------*/
   wxString s_meta ;
   if(    mtf::match( s_data,
                      R"=(<meta\M([^>]*?\mproperty=["']og:title["'].*?)>)=",
                      &s_meta
                    ) <= 1
       || mtf::match( s_meta, R"=(\mcontent=["']([^"']*)["'])=",
                      &s_name
                    ) <= 1
     )
   {  /*-------------------------------------------------------------------*/
      if( p_s_err_msg != nullptr )
      {  *p_s_err_msg = _( "The channel name could not be found" ) ; }
      /*-------------------------------------------------------------------*/
      return( -1 ) ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   s_name = s_name.Trim() ;
   /*----------------------------------------------------------------------*/
   return( 0 ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CChannelLive::hls_extract_streams( const wxString &s_url_m3u,
                                       bool boo_reverse
                                     )
{
   /*----------------------------------------------------------------------*/
   m_vec_stream.clear() ;
   /*--( Download the m3u )------------------------------------------------*/
   wxString s_data ;
   wxString s_last_url ;
   int i_err = mtf::download( s_url_m3u, s_data, &s_last_url ) ;
   if( i_err != CDownloadParam::DE_NO_ERROR_DONE )
   {  set_gs_error( GSERR_RETRY_LATER ) ; return( -1 ) ; }
   /*----------------------------------------------------------------------*/
   wxStringInputStream is( s_data ) ;
   wxTextInputStream   tis( is ) ;
   wxString            s_line ;
   int                 i_line_num = 0 ;
   wxString            s_url_video = s_last_url.BeforeLast( '/' ) ;
   /*----------------------------------------------------------------------*/
   while( !tis.GetInputStream().Eof() )
   {
      /*-------------------------------------------------------------------*/
      wxString s_name ;
      wxString s_url ;
      /*-------------------------------------------------------------------*/
      while(    !tis.GetInputStream().Eof()
             && ( s_line = mtf::read_m3u_line( tis, i_line_num )
                ).StartsWith( '#' )
           )
      {  /*----------------------------------------------------------------*/
         if( s_line.StartsWith( "#EXT-X-STREAM-INF:" ) )
         {  mtf::match( s_line, R"=([^\w]RESOLUTION=([^,]*))=", &s_name ) ; }
         else /*--( Used by Twitch )---------------------------------------*/
         if( s_line.StartsWith( "#EXT-X-MEDIA:" ) )
         {  mtf::match( s_line, R"=(,NAME="([^"]*)")=", &s_name ) ; }
         /*----------------------------------------------------------------*/
      }
      /*--( We should have found the URL )---------------------------------*/
      if( !tis.GetInputStream().Eof() )
      {  /*--( The URL may be relative )-----------------------------------*/
         if( !s_line.StartsWith( "http" ) ) { s_url = s_url_video + '/' ; }
         /*----------------------------------------------------------------*/
         s_url += s_line ;
         /*--( Each stream quality is accepted only once )-----------------*/
         if(    !s_name.empty()
             && !s_url.empty()
             && std::find( m_vec_stream.begin(), m_vec_stream.end(), s_name
                         ) == m_vec_stream.end()
           )
         {  m_vec_stream.emplace_back( s_name, s_url ) ; }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }
   /*--( The streams are sorted in "low -> high". We want the opposite )---*/
   if( boo_reverse )
   {  std::reverse( m_vec_stream.begin(), m_vec_stream.end() ) ; }
   /*--( Add the "HLS" stream at the first place )-------------------------*/
   m_vec_stream.insert( m_vec_stream.begin(),
                        str_stream( _( "auto" ), s_url_m3u )
                      ) ;
   /*----------------------------------------------------------------------*/
   return( 0 ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CChannelDailymotion::get_vlc_init() const
{  return( wxGetApp().M_s_vlc_init_dm.get() ) ; }

/*-------------------------------------------------------------------------*/
int CChannelDailymotion::st_extract_data_from_url( const wxString &co_s_url,
                                                   wxString &s_id,
                                                   wxString &s_name,
                                                   wxString *p_s_err_msg
                                                 )
{
   /*--( The given URL may be a shortened one )----------------------------*/
   wxString s_url = co_s_url ;
   if( mtf::unshort_url( s_url ) != 0 ) { return( -1 ) ; }

   /*--( The channel id is the last element of the url )-------------------*/
   s_id = s_url.AfterLast( '/' ) ;
   if( s_id.empty() )
   {  /*-------------------------------------------------------------------*/
      if( p_s_err_msg != nullptr )
      {  *p_s_err_msg = _( "The URL should end with the channel id" ) ; }
      /*-------------------------------------------------------------------*/
      return( -2 ) ;
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
   wxString s_data ;
   int i_err = mtf::download( s_url, s_data ) ;
   /*----------------------------------------------------------------------*/
   if( i_err != CDownloadParam::DE_NO_ERROR_DONE )
   {  /*-------------------------------------------------------------------*/
      if( p_s_err_msg != nullptr )
      {  *p_s_err_msg = _( "Problem downloading data" ) ; }
      /*-------------------------------------------------------------------*/
      return( -3 ) ;
      /*-------------------------------------------------------------------*/
   }

   /*--( One way to check if the channel exists ... )----------------------*/
   if( s_data.find( "<meta property=\"og:url\"" ) == wxString::npos )
   {  /*-------------------------------------------------------------------*/
      if( p_s_err_msg != nullptr ) { *p_s_err_msg = _( "Unknown channel" ) ;}
      /*-------------------------------------------------------------------*/
      return( -4 ) ;
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
   if( st_extract_name_from_page( s_data, s_name, p_s_err_msg ) != 0 )
   {  return( -5 ) ; }
   /*--( Clean name )------------------------------------------------------*/
   s_name.EndsWith( " - Dailymotion", &s_name ) ;
   /*----------------------------------------------------------------------*/
   return( 0 ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CChannelDailymotion::get_all_streams( CVlcMedia *p_media )
{
   /*--( Useful to go further ? )------------------------------------------*/
   if( !m_vec_stream.empty() ) { return( 0 ) ; }
   /*----------------------------------------------------------------------*/
   set_gs_error( GSERR_NO_ERROR ) ;
   /*----------------------------------------------------------------------*/
   wxBusyCursor tell_user_we_are_working ;
   message_get_all_streams( p_media ) ;
   /*----------------------------------------------------------------------*/
   wxString s_data ;
   int i_err = mtf::download( get_url(), s_data ) ;
   if( i_err != CDownloadParam::DE_NO_ERROR_DONE )
   {  set_gs_error( GSERR_DOWNLOAD ) ; return( -1 ) ; }

   /*--( Looking for the "auto" stream )-----------------------------------*/
   wxString s_url_m3u ;
   if( mtf::match( s_data,
                   R"=("qualities":{"auto":\[{)="
                   R"=("type":"application\\/x-mpegURL","url":"([^"]*)")=",
                   &s_url_m3u
                 ) <= 1
     )
   {  set_gs_error( GSERR_PARSING ) ; return( -2 ) ; }
   /*----------------------------------------------------------------------*/
   s_url_m3u.Replace( "\\/", "/", true ) ;
   /*----------------------------------------------------------------------*/
   if( hls_extract_streams( s_url_m3u, true ) != 0 ) { return( -3 ) ; }
   /*----------------------------------------------------------------------*/
   return( 0 ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CChannelTwitch::get_vlc_init() const
{  return( wxGetApp().M_s_vlc_init_tw.get() ) ; }

/*-------------------------------------------------------------------------*/
wxString CChannelTwitch::url_encode( const wxString &s )
{
   /*----------------------------------------------------------------------*/
   wxString s_res ;
   /*----------------------------------------------------------------------*/
   for( const wxUniChar &c : s )
   {
      /*-------------------------------------------------------------------*/
      if( c == '\n' ) { s_res += "%0D%0A" ; }
      else
      if( c == ' ' ) { s_res += '+' ; }
      else
      if( !wxIsalnum( c ) && c != '-' && c != '_' && c != '.' && c != '~' )
      {  s_res += wxString::Format( "%%%02X", c.GetValue() ) ; }
      else
      {  s_res += c ; }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return( s_res ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CChannelTwitch::st_extract_data_from_url( const wxString &co_s_url,
                                              wxString &s_id,
                                              wxString &s_name,
                                              wxString *p_s_err_msg
                                            )
{
   /*--( The given URL may be a shortened one )----------------------------*/
   wxString s_url = co_s_url ;
   if( mtf::unshort_url( s_url ) != 0 ) { return( -1 ) ; }
   /*--( Some twitch services accept the id only lowercase )---------------*/
   s_id = s_url.AfterLast( '/' ).Lower() ;
   /*----------------------------------------------------------------------*/
   if( s_id.empty() )
   {  /*-------------------------------------------------------------------*/
      if( p_s_err_msg != nullptr )
      {  *p_s_err_msg = _( "The URL should end with the channel id" ) ; }
      /*-------------------------------------------------------------------*/
      return( -2 ) ;
      /*-------------------------------------------------------------------*/
   }
   /*--( Check channel existence )-----------------------------------------*/
   e_gs_error gs_error = st_get_token_sig( s_id, nullptr, nullptr ) ;
   if( gs_error != GSERR_NO_ERROR )
   {  /*-------------------------------------------------------------------*/
      if( p_s_err_msg != nullptr )
      {  *p_s_err_msg = st_get_gs_error_message( gs_error ) ; }
      /*-------------------------------------------------------------------*/
      return( -3 ) ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   wxString s_data ;
   int i_err = mtf::download( s_url, s_data ) ;
   /*----------------------------------------------------------------------*/
   if( i_err != CDownloadParam::DE_NO_ERROR_DONE )
   {  /*-------------------------------------------------------------------*/
      if( p_s_err_msg != nullptr )
      {  *p_s_err_msg = st_get_gs_error_message( GSERR_DOWNLOAD ) ; }
      /*-------------------------------------------------------------------*/
      return( -4 ) ;
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
   if( st_extract_name_from_page( s_data, s_name, p_s_err_msg ) != 0 )
   {  return( -5 ) ; }
   /*--( Clean name )------------------------------------------------------*/
   s_name.EndsWith( " - Twitch", &s_name ) ;
   /*----------------------------------------------------------------------*/
   return( 0 ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CChannel::e_gs_error CChannelTwitch::st_get_token_sig( const wxString &s_id,
                                                       wxString *p_s_token,
                                                       wxString *p_s_sig
                                                     )
{
   /*----------------------------------------------------------------------*/
   const char *co_p_c_url_fmt
       = "http://api.twitch.tv/api/channels/%s/access_token?client_id=%s" ;
   const wxString co_s_url
       = wxString::Format( co_p_c_url_fmt, s_id,
                           wxGetApp().M_s_twitch_client_id.get()
                         ) ;
   /*----------------------------------------------------------------------*/
   wxString s_data ;
   int i_err = mtf::download( co_s_url, s_data ) ;
   /*----------------------------------------------------------------------*/
   if( i_err == CDownloadParam::DE_FILE_NOT_FOUND )
   {  return( GSERR_UNKNOWN_CHANNEL ) ; }
   if( i_err != CDownloadParam::DE_NO_ERROR_DONE )
   {  return( GSERR_DOWNLOAD ) ; }

   /*--( Extract token and sig )-------------------------------------------*/
   if( mtf::match( s_data, R"=("token":"(.*)","sig":"([^"]*)")=",
                   p_s_token, p_s_sig
                 ) <= 1
     )
   {  return( GSERR_PARSING ) ; }
   /*----------------------------------------------------------------------*/
   if( p_s_token != nullptr ) { p_s_token->Replace( "\\", "" ) ; }
   /*----------------------------------------------------------------------*/
   return( GSERR_NO_ERROR ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CChannelTwitch::get_all_streams( CVlcMedia *p_media )
{
   /*--( Useful to go further ? )------------------------------------------*/
   if(    m_boo_getting_streams
       || (   !m_vec_stream.empty()
            && m_ti_expiration != 0
            && time( nullptr ) + 10 < m_ti_expiration
          )
     )
   {  return( 0 ) ; }
   /*----------------------------------------------------------------------*/
   set_gs_error( GSERR_NO_ERROR ) ;
   /*----------------------------------------------------------------------*/
   wxBusyCursor tell_user_we_are_working ;
   CBoolInScope bool_scope( m_boo_getting_streams ) ;
   message_get_all_streams( p_media ) ;
   /*----------------------------------------------------------------------*/
   m_vec_stream.clear() ;
   /*----------------------------------------------------------------------*/
   wxString s_token ;
   wxString s_sig ;

   /*--( Get the "base" information before extracting the streams )--------*/
   set_gs_error( st_get_token_sig( m_s_id, &s_token, &s_sig ) ) ;
   if( get_gs_error() != GSERR_NO_ERROR ) { return( -1 ) ; }
   /*--( Streams expiration date & time )----------------------------------*/
   wxString s_ti_expiration ;
   if( mtf::match( s_token, R"=("expires":(\d*))=", &s_ti_expiration ) <= 1 )
   {  set_gs_error( GSERR_PARSING ) ; return( -2 ) ; }
   /*----------------------------------------------------------------------*/
   m_ti_expiration = wxAtol( s_ti_expiration ) ;

   /*--( Streams )---------------------------------------------------------*/
   const char *co_p_c_url_fmt
       = "http://usher.ttvnw.net/api/channel/hls/%s.m3u8?token=%s&sig=%s"
         "&allow_source=true&allow_spectre=true" ;
   const wxString co_s_url_m3u
       = wxString::Format( co_p_c_url_fmt, m_s_id, url_encode( s_token ),
                           s_sig
                         ) ;
   /*----------------------------------------------------------------------*/
   if( hls_extract_streams( co_s_url_m3u ) != 0 ) { return( -3 ) ; }
   /*----------------------------------------------------------------------*/
   return( 0 ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CChannelYoutube::get_vlc_init() const
{  return( wxGetApp().M_s_vlc_init_yt.get() ) ; }

/*-------------------------------------------------------------------------*/
int CChannelYoutube::st_extract_data_from_url( const wxString &co_s_url,
                                               wxString &s_id,
                                               wxString &s_name,
                                               wxString *p_s_err_msg
                                             )
{
   /*----------------------------------------------------------------------*/
   wxString s_data ;
   int i_err = mtf::download( co_s_url, s_data ) ;
   /*----------------------------------------------------------------------*/
   if( i_err != CDownloadParam::DE_NO_ERROR_DONE )
   {  /*-------------------------------------------------------------------*/
      if( p_s_err_msg != nullptr )
      {  *p_s_err_msg = _( "Problem downloading data" ) ; }
      /*-------------------------------------------------------------------*/
      return( -2 ) ;
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
   if(    mtf::match( s_data,
                      R"=(<meta itemprop="channelId" content="([^"]*)">)=",
                      &s_id
                    ) <= 1
       || s_id.empty()
     )
   {  /*-------------------------------------------------------------------*/
      if( p_s_err_msg != nullptr ) { *p_s_err_msg = _( "Unknown channel" ) ;}
      /*-------------------------------------------------------------------*/
      return( -3 ) ;
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
   if( st_extract_name_from_page( s_data, s_name, p_s_err_msg ) != 0 )
   {  return( -4 ) ; }
   /*----------------------------------------------------------------------*/
   return( 0 ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CChannelYoutube::get_all_streams( CVlcMedia *p_media )
{
   /*--( Useful to go further ? )------------------------------------------*/
   if( !m_vec_stream.empty() ) { return( 0 ) ; }
   /*----------------------------------------------------------------------*/
   set_gs_error( GSERR_NO_ERROR ) ;
   /*----------------------------------------------------------------------*/
   wxBusyCursor tell_user_we_are_working ;
   message_get_all_streams( p_media ) ;

   /*--( Load the live stream permanent link to find current channel id )--*/
   wxString s_url = get_url() ;
   wxString s_data ;
   int i_err = mtf::download( s_url, s_data, nullptr, 1 ) ;
   /*----------------------------------------------------------------------*/
   if( i_err != CDownloadParam::DE_NO_ERROR_DONE )
   {  set_gs_error( GSERR_DOWNLOAD ) ; return( -1 ) ; }

   /*-----------------------------------------------------------------------+
   ! Extract the temporary (current) live channel id                        !
   ! "{m,n}?" is used because "+?" seems to have a problem                  !
   +-----------------------------------------------------------------------*/
   wxString s_video_id ;
   if( mtf::match( s_data, R"=(/vi/([^/].{2,50}?)/.{0,20}?live.jpg)=",
                   &s_video_id
                 ) <= 1
     )
   {  set_gs_error( GSERR_RETRY_LATER ) ; return( -2 ) ; }

   /*--( Extract the display name of the channel and check its existence )-*/
   s_url = "https://youtube.com/get_video_info?video_id=" + s_video_id ;
   i_err = mtf::download( s_url, s_data ) ;
   if( i_err != CDownloadParam::DE_NO_ERROR_DONE )
   {  set_gs_error( GSERR_DOWNLOAD ) ; return( -3 ) ; }

   /*--( Check youtube error code )----------------------------------------*/
   if( mtf::match( s_data, R"=(\mstatus=fail\M)=" ) == 1 )
   {  /*--( Extract error code )-------------------------------------------*/
      wxString s_err_code ;
      if( mtf::match( s_data, R"=(\merrorcode=(\d+?)\M)=", &s_err_code) <= 1)
      {  set_gs_error( GSERR_PARSING ) ; return( -4 ) ; }
      /*--( Video can only be watched on Youtube ? )-----------------------*/
      if( s_err_code == "150" )
      {  set_gs_error( GSERR_NOT_VIEWABLE_HERE ) ; }
      else
      {  set_gs_error( GSERR_UNHANDLED ) ; }
      /*-------------------------------------------------------------------*/
      return( -5 ) ;
      /*-------------------------------------------------------------------*/
   }

   /*--( Looking for the m3u url )-----------------------------------------*/
   wxString s_url_m3u ;
   if( mtf::match( s_data, R"=(\&hlsvp=([^&]*)\&)=", &s_url_m3u ) <= 1 )
   {  set_gs_error( GSERR_PARSING ) ; return( -6 ) ; }

   /*--( The value is "escaped" : %3A%2F%2F => :// )-----------------------*/
   for( int i_num = 0 ;
        i_num < 5 && s_url_m3u.find( '%' ) != wxString::npos ;
        ++i_num
      )
   {  /*-------------------------------------------------------------------*/
      char *p_c_url_m3u = curl_unescape( s_url_m3u.ToAscii(), 0 ) ;
      s_url_m3u = p_c_url_m3u ;
      curl_free( p_c_url_m3u ) ;
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   if( hls_extract_streams( s_url_m3u, true ) != 0 ) { return( -5 ) ; }
   /*----------------------------------------------------------------------*/
   return( 0 ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
wxString CChannelUstream::get_vlc_init() const
{  return( wxGetApp().M_s_vlc_init_us.get() ) ; }

/*-------------------------------------------------------------------------*/
int CChannelUstream::st_extract_data_from_url( const wxString &co_s_url,
                                               wxString &s_id,
                                               wxString &s_name,
                                               wxString *p_s_err_msg
                                             )
{
   /*----------------------------------------------------------------------*/
   wxString s_data ;
   int i_err = mtf::download( co_s_url, s_data ) ;
   /*----------------------------------------------------------------------*/
   if( i_err != CDownloadParam::DE_NO_ERROR_DONE )
   {  /*-------------------------------------------------------------------*/
      if( p_s_err_msg != nullptr )
      {  *p_s_err_msg = _( "Problem downloading data" ) ; }
      /*-------------------------------------------------------------------*/
      return( -2 ) ;
      /*-------------------------------------------------------------------*/
   }

   /*--( Extract channel id )----------------------------------------------*/
   if( mtf::match( s_data,
                   R"=(<meta name="ustream:channel_id" content="([^"]*)">)=",
                   &s_id
                 ) <= 1
     )
   {  /*-------------------------------------------------------------------*/
      if( p_s_err_msg != nullptr ) { *p_s_err_msg = _( "Unknown channel" ) ;}
      /*-------------------------------------------------------------------*/
      return( -3 ) ;
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
   if( st_extract_name_from_page( s_data, s_name, p_s_err_msg ) != 0 )
   {  return( -4 ) ; }
   /*----------------------------------------------------------------------*/
   return( 0 ) ;
   /*----------------------------------------------------------------------*/
}

/*--------------------------------------------------------------------------+
! The HLS url seems "constant" but to get the username (to build the        !
! browser url) and avoid a maybe variable HLS url, the Ustream api is used  !
+--------------------------------------------------------------------------*/
int CChannelUstream::get_all_streams( CVlcMedia *p_media )
{
   /*--( Useful to go further ? )------------------------------------------*/
   if( !m_vec_stream.empty() ) { return( 0 ) ; }
   /*----------------------------------------------------------------------*/
   set_gs_error( GSERR_NO_ERROR ) ;
   /*----------------------------------------------------------------------*/
   wxBusyCursor tell_user_we_are_working ;
   message_get_all_streams( p_media ) ;

   /*--( Loading the HLS and filling username )----------------------------*/
   wxString s_url = "https://api.ustream.tv/channels/" + m_s_id + ".json" ;
   wxString s_data ;
   /*----------------------------------------------------------------------*/
   int i_err = mtf::download( s_url, s_data ) ;
   if( i_err != CDownloadParam::DE_NO_ERROR_DONE )
   {  set_gs_error( GSERR_DOWNLOAD ) ; return( -1 ) ; }
   /*--( Username is used to build the main url )--------------------------*/
   if( mtf::match( s_data, R"=("username":"([^"]*)")=", &m_s_username ) <= 1)
   {  set_gs_error( GSERR_PARSING ) ; return( -2 ) ; }
   /*----------------------------------------------------------------------*/
   wxString s_url_m3u ;
   if( mtf::match( s_data, R"=("hls":"([^"]*)")=", &s_url_m3u ) <= 1 )
   {  set_gs_error( GSERR_PARSING ) ; return( -3 ) ; }
   /*----------------------------------------------------------------------*/
   s_url_m3u.Replace( "\\/", "/", true ) ;
   /*----------------------------------------------------------------------*/
   if( hls_extract_streams( s_url_m3u ) != 0 ) { return( -4 ) ; }
   /*----------------------------------------------------------------------*/
   return( 0 ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                         End of file CChannel.cpp                          !
+==========================================================================*/
