/*====================================+=====================================+
! File CDlgNewVersion.cpp             ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include <wx/sstream.h>
#include <wx/txtstrm.h>
#include "CApp.h"
#include "CDlgVersion.h"
#include "CDownload.h"
#include "CFrame.h"
#include "CDlgInstall.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
CDlgVersion::CDlgVersion( wxWindow *parent,
                          const wxString &s_main_info,
                          const wxString &s_data
                        )
           : wxDialog( parent, wxID_ANY, _( "Version" ),
                       wxDefaultPosition, wxDefaultSize,
                       wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER
                     ),
             m_s_data( s_data )
{
   /*----------------------------------------------------------------------*/
   build_ui() ;
   /*----------------------------------------------------------------------*/
   m_bt_web_site->Bind( wxEVT_COMMAND_BUTTON_CLICKED,
                        &CDlgNewVersion::OnBtGotoWebSite, this
                      ) ;
   m_bt_close->Bind( wxEVT_COMMAND_BUTTON_CLICKED,
                     &CDlgNewVersion::OnBtClose, this
                   ) ;
   /*----------------------------------------------------------------------*/
   m_stx_main_info->SetLabel( s_main_info ) ;

   /*--( Parse version data )----------------------------------------------*/
   wxStringInputStream is( s_data ) ;
   wxTextInputStream   tis( is )    ;
   wxString            s_line       ;
   wxString            s_platform   ;
   wxString            s_url        ;

   /*--( The first line is the version number : skip it )------------------*/
   if( is.Eof() || tis.ReadLine().empty() )
   {  return ; }
   /*--( Then come the platforms/urls )------------------------------------*/
   while( !is.Eof() && !( s_line = tis.ReadLine() ).empty() )
   {  /*--( The two elements are separated by one or more spaces )---------*/
      s_platform = s_line.BeforeFirst( ' ', &s_url ) ;
      /*--( Remove leading spaces (the txt file is clearer ... ) )---------*/
      m_ms_url[ s_platform ] = s_url.Trim( false ) ;
      /*-------------------------------------------------------------------*/
   }
   /*--( An empty line separates urls from version history )---------------*/
   if( is.Eof() ) { return ; }
   /*--( The rest is history ... )-----------------------------------------*/
   m_txt_version_history->SetValue( s_data.Mid( is.TellI() ) ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CDlgVersion::OnBtGotoWebSite( wxCommandEvent & WXUNUSED( event ) )
{
   /*----------------------------------------------------------------------*/
   wxCommandEvent event_gtws( wxEVT_MENU, ID_WEB_SITE_DOWNLOAD ) ;
   wxGetApp().m_frame->GetEventHandler()->ProcessEvent( event_gtws ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CDlgVersion::get_modal()
{
   /*----------------------------------------------------------------------*/
   return( ShowModal() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CDlgVersion::build_ui()
{
   /*----------------------------------------------------------------------*/
   SetSizeHints( wxSize( 600, 400 ), wxDefaultSize ) ;
   /*----------------------------------------------------------------------*/
   wxBoxSizer *bsz_gen = new wxBoxSizer( wxVERTICAL ) ;
   /*----------------------------------------------------------------------*/
     m_stx_main_info
   = new wxStaticText( this, wxID_ANY, wxEmptyString,
                       wxDefaultPosition, wxDefaultSize,
                       wxALIGN_CENTRE_HORIZONTAL | wxST_NO_AUTORESIZE
                     ) ;
   m_stx_main_info->SetFont( m_stx_main_info->GetFont().Scale( 2 ).Bold() ) ;
   bsz_gen->Add( m_stx_main_info, 0, wxEXPAND | wxTOP | wxBOTTOM, 20 ) ;
   /*----------------------------------------------------------------------*/
   wxStaticBoxSizer *stbsz_hist
   = new wxStaticBoxSizer( wxVERTICAL, this, _( "Version history" ) ) ;
   wxStaticBox *stb_hist = stbsz_hist->GetStaticBox() ;
   m_txt_version_history
   = new wxTextCtrl( stb_hist, wxID_ANY, wxEmptyString,
                     wxDefaultPosition, wxDefaultSize,
                     wxHSCROLL | wxTE_MULTILINE | wxTE_READONLY
                   ) ;
   stbsz_hist->Add( m_txt_version_history, 1, wxEXPAND, 5 ) ;
   bsz_gen->Add( stbsz_hist, 1, wxLEFT | wxRIGHT | wxEXPAND, 5 ) ;
   /*----------------------------------------------------------------------*/
   m_bsz_btn = new wxBoxSizer( wxHORIZONTAL ) ;

   m_bt_web_site = new wxButton( this, wxID_ANY, _( "&Web site" ) ) ;
   m_bsz_btn->Add( m_bt_web_site, 0, wxLEFT | wxRIGHT, 5 ) ;

   m_bt_close = new wxButton( this, wxID_CLOSE, wxEmptyString ) ;
   m_bt_close->SetDefault() ;
   m_bsz_btn->Add( m_bt_close, 0, wxLEFT | wxRIGHT, 5 ) ;

   bsz_gen->Add( m_bsz_btn, 0, wxALIGN_RIGHT | wxALL, 10 ) ;
   /*----------------------------------------------------------------------*/
   SetSizer( bsz_gen ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CDlgLastVersion::CDlgLastVersion( wxWindow *parent,
                                  const wxString &s_main_info,
                                  const wxString &s_data
                                )
               : CDlgVersion( parent, s_main_info, s_data )
{
   /*----------------------------------------------------------------------*/
   mtf::init_tlw_rect( this, wxGetApp().M_lastversion_rec ) ;
   Layout() ;
   /*----------------------------------------------------------------------*/
   m_bt_close->SetFocus() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CDlgLastVersion::~CDlgLastVersion()
{
   /*----------------------------------------------------------------------*/
   mtf::save_tlw_rect( this, wxGetApp().M_lastversion_rec ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CDlgNewVersion::CDlgNewVersion( wxWindow *parent,
                                const wxString &s_main_info,
                                const wxString &s_data
                              )
              : CDlgVersion( parent, s_main_info, s_data )
{
   /*----------------------------------------------------------------------*/
   m_bt_download = new wxButton( this, wxID_ANY, _( "&Download" ) ) ;
   m_bsz_btn->Prepend( m_bt_download, 0, wxLEFT | wxRIGHT, 5 ) ;

     m_bt_download_install
   = new wxButton( this, wxID_ANY, _( "Download && &Install" ) ) ;
   m_bt_download_install->SetDefault();
   m_bsz_btn->Prepend( m_bt_download_install, 0, wxLEFT | wxRIGHT, 5 ) ;

   /*----------------------------------------------------------------------*/
   m_bt_download_install->Bind( wxEVT_COMMAND_BUTTON_CLICKED,
                                &CDlgNewVersion::OnBtDownloadInstall, this
                              ) ;
   m_bt_download->Bind( wxEVT_COMMAND_BUTTON_CLICKED,
                        &CDlgNewVersion::OnBtDownload, this
                      ) ;
   /*----------------------------------------------------------------------*/
   mtf::init_tlw_rect( this, wxGetApp().M_newversion_rec ) ;
   Layout() ;
   /*----------------------------------------------------------------------*/
   m_bt_download_install->SetFocus() ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CDlgNewVersion::~CDlgNewVersion()
{
   /*----------------------------------------------------------------------*/
   mtf::save_tlw_rect( this, wxGetApp().M_newversion_rec ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CDlgNewVersion::OnBtDownload( wxCommandEvent &event )
{
   /*--( After download only a "wait" is done )----------------------------*/
   download( wxFileName(), CDlgDownload::DDO_END_WAIT ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
void CDlgNewVersion::OnBtDownloadInstall( wxCommandEvent &event )
{
   /*----------------------------------------------------------------------*/
   if( wxMessageBox( _( "Start download and installation ?" ),
                     _( "Confirmation" ), wxYES_NO
                   ) != wxYES
     )
   {  return ; }
   /*----------------------------------------------------------------------*/
   wxFileName fn_exe_path = wxFileName::DirName( wxGetApp().get_exe_path() );
   /*--( By default the new version is downloaded in the executable path )-*/
   if( download( fn_exe_path ) == CDownloadParam::DE_NO_ERROR_DONE )
   {
       /*--( If the installation failed no restart is needed )-------------*/
       if( install() != CInstallThread::IE_NO_ERROR_DONE )
       {  wxGetApp().no_start_on_exit() ; }
       else
       {  EndModal( 0 ) ; }
       /*------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CDlgNewVersion::download( const wxFileName &fn_local,
                              CDlgDownload::e_option ddo_option
                            )
{
   /*--( Look for the right URL )------------------------------------------*/
   auto cit = m_ms_url.find( MTF_BUILT_TYPE ) ;
   if( cit == m_ms_url.end() )
   {  wxLogError( _( "The URL for your system has not been found (%s)" ),
                  MTF_BUILT_TYPE
                ) ;
      return( -1 ) ;
   }
   /*--( Use specified directory of ask the user to choose it )------------*/
   if( fn_local.IsOk() )
   {  m_fn_local = fn_local ; }
   else
   {  /*--( Choose directory/name )----------------------------------------*/
      wxFileDialog dlg_fd( this, "", "", cit->second.AfterLast( '/' ),
                           wxFileSelectorDefaultWildcardStr,
                           wxFD_SAVE | wxFD_OVERWRITE_PROMPT
                         ) ;
      if( dlg_fd.ShowModal() == wxID_CANCEL )
      {  return( -2 ) ; }
      /*-------------------------------------------------------------------*/
      m_fn_local = dlg_fd.GetPath() ;
      /*-------------------------------------------------------------------*/
   }

   /*--( Lets go )---------------------------------------------------------*/
   CDlgDownload dlg( this, cit->second, m_fn_local, ddo_option ) ;
   m_fn_local = dlg.get_param().build_local_fullpath() ;
   return( dlg.go() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
int CDlgNewVersion::install()
{
   /*----------------------------------------------------------------------*/
   return( CDlgInstall( this, m_fn_local ).go() ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                      End of file CDlgNewVersion.cpp                       !
+==========================================================================*/
