/*====================================+=====================================+
! File CVlcInterface.cpp              ! Copyright (C) 2016-2018 Remi PASCAL !
+-------------------------------------+-------------------------------------+
! This file is part of matelefree.                                          !
! matelefree is free software: you can redistribute it and/or modify it     !
! under the terms of the GNU General Public License as published by the     !
! Free Software Foundation, either version 3 of the License, or any later   !
! version.                                                                  !
! matelefree is distributed in the hope that it will be useful, but WITHOUT !
! ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or     !
! FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for !
! more details.                                                             !
! You should have received a copy of the GNU General Public License along   !
! with matelefree. If not, see <http://www.gnu.org/licenses/>.              !
+==========================================================================*/



/*-------------------------------------------------------------------------*/
#include <wx/wx.h>
#include <wx/tokenzr.h>
#include <wx/uri.h>
#include "CVlcInterface.h"
/*-------------------------------------------------------------------------*/



/*-------------------------------------------------------------------------*/
CVlcInterface::CVlcInterface( const wxString &s_vlc_path,
                              const wxString &s_vlc_init
                            )
{
   /*----------------------------------------------------------------------*/
   wxString s_path = s_vlc_path ;
   /*----------------------------------------------------------------------*/
   #ifdef __WXMSW__
      /*--( By default the path to vlc executable is used )----------------*/
      if( s_path.empty() )
      {  s_path = wxGetApp().m_fn_vlc.GetPath() ; }
      /*-------------------------------------------------------------------*/
      {  /*--( Suppress useless warnings )---------------------------------*/
         wxLogNull no_log ;
         /*--( Revert to standard search )---------------------------------*/
         ::SetDllDirectory( NULL ) ;
         /*--( Set directory to load "sub" dlls loaded )-------------------*/
         ::SetDllDirectory( s_path ) ;
         /*--( Force path else if already loaded it won't work )-----------*/
         Load( wxFileName( s_path, "libvlc.dll" ).GetFullPath() ) ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   #elif defined( __WXGTK__ )
      /*--( libvlc can have different names ... )--------------------------*/
      wxString tb_s_libvlc[] = { "libvlc.so", "libvlc.so.5" } ;
      /*-------------------------------------------------------------------*/
      {
         /*----------------------------------------------------------------*/
         wxLogNull no_log ; // Suppress useless warnings
         /*----------------------------------------------------------------*/
         for( int i_num = 0 ;
              i_num < ( int )WXSIZEOF( tb_s_libvlc )
           && !Load( wxFileName( s_path, tb_s_libvlc[ i_num ] ).GetFullPath()
                   ) ;
              ++i_num
            )
         {  ; }
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   #else
      #error Unmanaged platform to: load libvlc
   #endif
   /*----------------------------------------------------------------------*/
   if( !IsLoaded() )
   {  return ; }
   /*----------------------------------------------------------------------*/
   {  wxLogNull no_log ; // Suppress useless warnings
      /*-------------------------------------------------------------------*/
      INIT_P_FNC( libvlc_new ) ;
      INIT_P_FNC( VLC_Version ) ;
      INIT_P_FNC( libvlc_get_version ) ;
      INIT_P_FNC( libvlc_set_user_agent ) ;
      INIT_P_FNC( libvlc_set_app_id ) ;
      INIT_P_FNC( libvlc_media_player_new ) ;
      INIT_P_FNC( libvlc_media_player_event_manager ) ;
      INIT_P_FNC( libvlc_media_event_manager ) ;
      INIT_P_FNC( libvlc_media_add_option ) ;
      INIT_P_FNC( libvlc_event_attach ) ;
      INIT_P_FNC( libvlc_event_detach ) ;
      INIT_P_FNC( libvlc_event_type_name ) ;
      INIT_P_FNC( libvlc_media_player_release ) ;
      INIT_P_FNC( libvlc_release ) ;
      INIT_P_FNC( libvlc_media_player_get_xwindow ) ;
      INIT_P_FNC( libvlc_media_player_set_xwindow ) ;
      INIT_P_FNC( libvlc_media_player_get_hwnd ) ;
      INIT_P_FNC( libvlc_media_player_set_hwnd ) ;
      INIT_P_FNC( libvlc_media_new_location ) ;
      INIT_P_FNC( libvlc_media_new_path ) ;
      INIT_P_FNC( libvlc_media_player_get_media ) ;
      INIT_P_FNC( libvlc_media_player_set_media ) ;
      INIT_P_FNC( libvlc_media_get_mrl ) ;
      INIT_P_FNC( libvlc_video_get_track_description ) ;
      INIT_P_FNC( libvlc_video_get_track_count ) ;
      INIT_P_FNC( libvlc_video_get_track ) ;
      INIT_P_FNC( libvlc_video_set_track ) ;
      INIT_P_FNC( libvlc_audio_get_track_description ) ;
      INIT_P_FNC( libvlc_audio_get_track_count ) ;
      INIT_P_FNC( libvlc_audio_get_track ) ;
      INIT_P_FNC( libvlc_audio_set_track ) ;
      INIT_P_FNC( libvlc_video_get_spu_description ) ;
      INIT_P_FNC( libvlc_video_get_spu_count ) ;
      INIT_P_FNC( libvlc_track_description_list_release ) ;
      INIT_P_FNC( libvlc_video_get_spu ) ;
      INIT_P_FNC( libvlc_video_set_spu ) ;
      INIT_P_FNC( libvlc_media_release ) ;
      INIT_P_FNC( libvlc_media_player_play ) ;
      INIT_P_FNC( libvlc_media_player_pause ) ;
      INIT_P_FNC( libvlc_media_player_stop ) ;
      INIT_P_FNC( libvlc_media_player_is_playing ) ;
      INIT_P_FNC( libvlc_media_player_get_state ) ;
      INIT_P_FNC( libvlc_media_get_stats ) ;
      INIT_P_FNC( libvlc_errmsg ) ;
      INIT_P_FNC( libvlc_audio_get_volume ) ;
      INIT_P_FNC( libvlc_audio_set_volume ) ;
      INIT_P_FNC( libvlc_audio_get_mute ) ;
      INIT_P_FNC( libvlc_audio_set_mute ) ;
      INIT_P_FNC( libvlc_get_fullscreen ) ;
      INIT_P_FNC( libvlc_set_fullscreen ) ;
      INIT_P_FNC( libvlc_video_take_snapshot ) ;
      INIT_P_FNC( libvlc_video_set_mouse_input ) ;
      INIT_P_FNC( libvlc_video_set_deinterlace ) ;
      INIT_P_FNC( libvlc_video_set_key_input ) ;
      /*-------------------------------------------------------------------*/
   }

   /*----------------------------------------------------------------------*/
   if( !is_version_ok() )
   {  return ; }

   /*--( Some parameters to pass to "libvlc" ? )---------------------------*/
   std::vector< const char * > vec_argv ;
   if( !s_vlc_init.empty() )
   {
      /*-------------------------------------------------------------------*/
      wxStringTokenizer stok_argv( s_vlc_init, " " ) ;
      while( stok_argv.HasMoreTokens() )
      {  vec_argv.push_back( wxStrdup( stok_argv.GetNextToken() ) ) ; }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   const char **p_p_c_argv = ( vec_argv.empty() ? nullptr : &vec_argv[ 0 ] );
   m_inst = m_libvlc_new( vec_argv.size(), p_p_c_argv ) ;
   /*--( The allocated data have to be deleted )---------------------------*/
   for( const char *p_c : vec_argv ) { delete p_c ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
CVlcInterface::~CVlcInterface()
{
   /*----------------------------------------------------------------------*/
   if( m_inst != nullptr ) { m_libvlc_release( m_inst ) ; }
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
const char *CVlcInterface::get_version() const
{
   /*----------------------------------------------------------------------*/
   return( m_libvlc_get_version != nullptr
           ? m_libvlc_get_version()
           : ( m_VLC_Version != nullptr ? m_VLC_Version() : nullptr )
         ) ;
   /*----------------------------------------------------------------------*/
}

/*--( Minimum version ok )-------------------------------------------------*/
static int st_tb_i_version_min[] = { 2, 0 } ;

/*-------------------------------------------------------------------------*/
const char *CVlcInterface::st_min_version_ok()
{
   /*--( Five digits plus one dot )----------------------------------------*/
   static char st_tb_c_version_min[ WXSIZEOF( st_tb_i_version_min ) * 6 + 2];
   /*--( It is useless to build it many times )----------------------------*/
   if( st_tb_c_version_min[ 0 ] == '\0' )
   {
      /*-------------------------------------------------------------------*/
      char *p_c_dot = st_tb_c_version_min ;
      /*-------------------------------------------------------------------*/
      for( size_t sz_num = 0 ; sz_num < WXSIZEOF( st_tb_i_version_min ) ;
           ++sz_num
         )
      {  /*----------------------------------------------------------------*/
         if( sz_num > 0 ) { *p_c_dot++ = '.' ; }
         /*----------------------------------------------------------------*/
         p_c_dot += sprintf( p_c_dot, "%d", st_tb_i_version_min[ sz_num ] ) ;
         /*----------------------------------------------------------------*/
      }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return( st_tb_c_version_min ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/
bool CVlcInterface::is_version_ok() const
{
   /*----------------------------------------------------------------------*/
   const char *p_c_version = get_version() ;
   /*----------------------------------------------------------------------*/
   if( p_c_version == nullptr ) { return( false ) ; }
   /*----------------------------------------------------------------------*/
   int tb_i_version[ WXSIZEOF( st_tb_i_version_min ) ] ;
   /*----------------------------------------------------------------------*/
   if( wxSscanf( p_c_version, "%d.%d.",
                 &tb_i_version[ 0 ], &tb_i_version[ 1 ]
               ) != 2
     )
   {  return( false ) ; }
   /*----------------------------------------------------------------------*/
   for( size_t sz_num = 0 ; sz_num < WXSIZEOF( tb_i_version ) ; ++sz_num )
   {
      /*-------------------------------------------------------------------*/
      if( tb_i_version[ sz_num ] != st_tb_i_version_min[ sz_num ] )
      {  return( tb_i_version[ sz_num ] > st_tb_i_version_min[ sz_num ] ) ; }
      /*-------------------------------------------------------------------*/
   }
   /*----------------------------------------------------------------------*/
   return( true ) ;
   /*----------------------------------------------------------------------*/
}

/*-------------------------------------------------------------------------*/



/*==========================================================================+
!                       End of file CVlcInterface.cpp                       !
+==========================================================================*/
